import os
#import jgtdotenv
from dotenv import load_dotenv
from dotenv import dotenv_values

load_dotenv()  # take environment variables from .env.
#env=load_dotenv(os.getenv(os.getcwd()))
env = dotenv_values(".env")




# print(FXCM_TOKEN_REST_API_DEMO)

local_fn_compression='gzip'


def get_pov_local_data_filename(instrument,timeframe,local_data_dir='./data',local_fn_suffix='.full.csv.gz'):
  print("-------#@STCIssue FIXING REQUIRED-----------------")
  tf=timeframe
  if tf=="m1":
    tf="mi1"
  return local_data_dir + "/"+ instrument.replace("/","-") + "_" + tf + local_fn_suffix

pysroot=os.getenv('pysroot')
#pysroot='/w/o/pys/'
CDS_URL_BASE=os.getenv('CDS_URL_BASE')
phurlbase=CDS_URL_BASE
#phurlbase='https://ai.guillaumeisabelle.com/sds/datasets/cds/' #todo rename as pds, cds are going to be indicators/cds data

DROPBOX_ETC_PATH= '/w/etc/'


if os.getenv('usereal') == 'True' :
  #@STCIssue FIXING REQUIRED
  print('TRADING REAL ACTIVATED')

def setreal():
  #@STCIssue FIXING REQUIRED
  print('TRADING REAL ACTIVATED')
  

def setdemo():
  #@STCIssue FIXING REQUIRED
  print('TRADING DEMO/PAPER ACTIVATED')
  


# %%
contextInstruments=['AUD/CAD',
 'AUD/JPY',
 'AUD/USD',
 'CAD/CHF',
 'CAD/JPY',
 'EUR/CAD',
 'EUR/USD',
 'GBP/CAD',
 'GBP/USD',
 'NAS100',
 'SPX500',
 'NZD/CAD',
 'NZD/USD',
 'USD/CAD'
 ]

contextTimeframes=['m1','m5','m15']
# contextTimeframes=['m1','m5','m15','H1','H4','D1','W1','M1']





from typing import Callable

from . import fxcorepy


class TableListener(fxcorepy.AO2GTableListener):
    """The class implements the abstract class AO2GTableListener and calls the passed functions on the appropriate trading tables events: adding/changing/deleting of rows and table status changes."""
    def __init__(self,
                 table: fxcorepy.O2GTable = None,
                 on_changed_callback: Callable[
                     [fxcorepy.AO2GTableListener, str, fxcorepy.O2GRow], None
                 ] = None,
                 on_added_callback: Callable[
                     [fxcorepy.AO2GTableListener, str, fxcorepy.O2GRow], None
                 ] = None,
                 on_deleted_callback: Callable[
                     [fxcorepy.AO2GTableListener, str, fxcorepy.O2GRow], None
                 ] = None,
                 on_status_changed_callback: Callable[
                     [fxcorepy.AO2GTableListener,
                      fxcorepy.O2GTableStatus], None] = None
                 ) -> None:
        """ The constructor.
        
            Parameters
            ----------
            table : 
                An instance of O2GTable.
            on_added_callback : O2GTable)= None, (typing.Callable[[AO2GTableListener, str, O2GRow], None]
                The function that is called when a row is added to the table.
            on_deleted_callback : typing.Callable[[AO2GTableListener, str, O2GRow], None]
                The function that is called when a row is deleted from the table.
            on_changed_callback : typing.Callable[[AO2GTableListener, str, O2GRow], None]
                The function that is called when a row in the table is changed.
            on_status_changed_callback : typing.Callable[[AO2GTableListener, O2GTableStatus], None]
                The function that is called when a table status is changed.
        
            Returns
            -------
            None
        
        """
        fxcorepy.AO2GTableListener.__init__(self)
        self._on_changed_callback = on_changed_callback
        self._on_added_callback = on_added_callback
        self._on_deleted_callback = on_deleted_callback
        self._on_status_changed_callback = on_status_changed_callback
        self._table = table

    def on_added(self, row_id: str, row: fxcorepy.O2GRow) -> None:  # native call
        """ Implements the method AO2GTableListener.on_added and calls the function that processes notifications about the row addition to a table. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_added_callback:
            self._on_added_callback(self, row_id, row)

    def on_changed(self, row_id: str, row: fxcorepy.O2GRow) -> None:  # native call
        """ Implements the method AO2GTableListener.on_changed and calls the function that processes notifications about the row change in a table. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_changed_callback:
            self._on_changed_callback(self, row_id, row)

    def on_deleted(self, row_id: str, row: fxcorepy.O2GRow) -> None:  # native call
        """ Implements the method AO2GTableListener.on_deleted and calls the function that processes notifications about the row deletion from a table. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_deleted_callback:
            self._on_deleted_callback(self, row_id, row)

    def on_status_changed(self, status: fxcorepy.O2GTableStatus) -> None:  # native call
        """ Implements the method AO2GTableListener.on_status_changed and calls the function that processes notifications about the table status change. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_status_changed_callback:
            self._on_status_changed_callback(self, status)

    def _unsubscribe(self, type_u):
        self._table.unsubscribe_update(type_u, self)

    def unsubscribe(self) -> None:
        """ Unsubscribes a table listener from table updates.
        
            Returns
            -------
            None
        
        """
        if self._table is not None:
            if self._on_changed_callback is not None:
                self._unsubscribe(fxcorepy.O2GTableUpdateType.UPDATE)
            if self._on_deleted_callback is not None:
                self._unsubscribe(fxcorepy.O2GTableUpdateType.DELETE)
            if self._on_added_callback is not None:
                self._unsubscribe(fxcorepy.O2GTableUpdateType.INSERT)
            if self._on_status_changed_callback is not None:
                self._table.unsubscribe_status(self)

    def subscribe(self, table: fxcorepy.O2GTable = None) -> None:
        """ Subscribes a table listener to updates of a certain table.
        
            Parameters
            ----------
            table : O2GTable
                An instance of O2GTable.
        
            Returns
            -------
            None
        
        """
        if self._table is None:
            if table is None:
                raise ValueError("Table is not set")
            self._table = table
        if self._on_changed_callback is not None:
            self._table.subscribe_update(fxcorepy.O2GTableUpdateType.UPDATE, self)
        if self._on_deleted_callback is not None:
            self._table.subscribe_update(fxcorepy.O2GTableUpdateType.DELETE, self)
        if self._on_added_callback is not None:
            self._table.subscribe_update(fxcorepy.O2GTableUpdateType.INSERT, self)
        if self._on_status_changed_callback is not None:
            self._table.subscribe_status(self)




# OFFERS_CSV_DATA="""18,CAD/JPY
# 1010,SPX500
# 2020,WHEATF
# 3008,2USNote
# 3,GBP/USD
# 36,EUR/NOK
# 14,EUR/AUD
# 3004,5USNote
# 17,AUD/JPY
# 32,EUR/SEK
# 21,GBP/NZD
# 2003,SOYF
# 1035,EUSTX50
# 1002,ESP35
# 89,NZD/CHF
# 96,EUR/HUF
# 1013,US30
# 3011,FED30D
# 4,USD/CHF
# 22,GBP/AUD
# 13,GBP/CHF
# 1001,AUS200
# 15,EUR/CAD
# 90,CAD/CHF
# 1012,UK100
# 4001,XAU/USD
# 3005,10USNote
# 39,AUD/CHF
# 91,NZD/CAD
# 2,USD/JPY
# 30,USD/SEK
# 5,EUR/CHF
# 38,USD/MXN
# 83,USD/TRY
# 3010,Schatz
# 1008,NAS100
# 97,USD/HUF
# 1,EUR/USD
# 20,GBP/CAD
# 16,AUD/CAD
# 1060,US2000
# 1004,GER30
# 12,CHF/JPY
# 50,USD/HKD
# 37,USD/NOK
# 2001,USOil
# 1058,USDOLLAR
# 1007,JPN225
# 98,TRY/JPY
# 71,ZAR/JPY
# 9,EUR/GBP
# 47,USD/ZAR
# 105,USD/CNH
# 4002,XAG/USD
# 2021,CORNF
# 6,AUD/USD
# 11,GBP/JPY
# 1003,FRA40
# 3001,Bund
# 2015,NGAS
# 2002,UKOil
# 8,NZD/USD
# 1005,HKG33
# 3012,EURIBOR3M
# 19,NZD/JPY
# 40,EUR/NZD
# 28,AUD/NZD
# 3014,SONIA3M
# 3009,Bobl
# 1016,Copper
# 87,EUR/TRY
# 7,USD/CAD
# 10,EUR/JPY"""

# def offer_id_to_instrument(offer_id):
#     for line in OFFERS_CSV_DATA.split("\n"):
#         if line.startswith(str(offer_id) + ","):
#             return line.split(",")[1]
#     return None

# def instrument_to_offer_id(symbol):
#     for line in OFFERS_CSV_DATA.split("\n"):
#         if line.split(",")[1] == symbol:
#             return int(line.split(",")[0])
#     return None